/**********
 * Copyright (c) 2004 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

// fixme replace with multi-screen prefs
// fixme allow subsystems to register with Prefs for change notices 
// (rather than hardcoding those here)

#include "includes.h"
#include "formutils.h"
#include "data/prefs.h"
#include "ssh/ssh.h"
#include "rsrc/rsrc.h"
#include "vt100/putty.h" // set_palm_color (fixme yuck)

#include "prefsform.h"


extern struct ssh_session_t *ss;

static void LoadPrefsForm(void) FORMS_SEGMENT;
static void SavePrefsForm(void) FORMS_SEGMENT;
static Boolean SampleTextEventHandler(struct FormGadgetTypeInCallback *gadget, 
                                      UInt16 cmd, void *paramP) FORMS_SEGMENT;
static void InvalidateSampleText(void) FORMS_SEGMENT;
static void DrawSampleText(RectangleType bounds) FORMS_SEGMENT;

static void LoadPrefsForm(void)
{
    uint32_t value;
    char *str;

    // font
    value = PrefsGetInt(prefTerminalFont, font4x6);
    if (value == font4x6) 
        PrvSetControlValue(PrefsFormSmallFontCheckboxID, 1);
    else if (value == font6x10) 
        PrvSetControlValue(PrefsFormLargeFontCheckboxID, 1);
    // else some future font size - ignore

    // colors
    value = PrefsGetInt(prefTerminalForeColor, blackColor);
    if (value < 8) {
        ListPtr lst = PrvGetObjectByID(PrefsFormForeColorListID);
        LstSetSelection(lst, value);
        PrvSetControlLabel(PrefsFormForeColorTriggerID, 
                           LstGetSelectionText(lst, value));
    }
    value = PrefsGetInt(prefTerminalBackColor, whiteColor);
    if (value < 8) {
        ListPtr lst = PrvGetObjectByID(PrefsFormBackColorListID);
        LstSetSelection(lst, value);
        PrvSetControlLabel(PrefsFormBackColorTriggerID, 
                           LstGetSelectionText(lst, value));
    }

    // term type
    str = PrefsGetString(prefTerminalType, "xterm");
    if (0 == strcasecmp(str, "xterm")) 
        PrvSetControlValue(PrefsFormXtermCheckboxID, 1);
    else if (0 == strcasecmp(str, "xterm-color")) 
        PrvSetControlValue(PrefsFormXtermColorCheckboxID, 1);
    else if (0 == strcasecmp(str, "vt100")) 
        PrvSetControlValue(PrefsFormVT100CheckboxID, 1);
    else if (0 == strcasecmp(str, "vt220")) 
        PrvSetControlValue(PrefsFormVT220CheckboxID, 1);
    // else some future termtype - ignore
    arena_free(str);
}


static void SavePrefsForm(void)
{
    uint32_t value;
    int selection;
    Boolean sshChanged = false;

    // font
    value = PrefsGetInt(prefTerminalFont, font4x6);
    if (PrvGetControlValue(PrefsFormSmallFontCheckboxID) && value != font4x6) {
        PrefsPutInt(prefTerminalFont, font4x6);
        sshChanged = true;
    } else if (PrvGetControlValue(PrefsFormLargeFontCheckboxID) && value != font6x10) {
        PrefsPutInt(prefTerminalFont, font6x10);
        sshChanged = true;
    }

    // color
    value = PrefsGetInt(prefTerminalForeColor, defaultForeColor);
    selection = LstGetSelection(PrvGetObjectByID(PrefsFormForeColorListID));
    if (selection != value  &&  selection != noListSelection) {
        PrefsPutInt(prefTerminalForeColor, selection);
        sshChanged = true;
    }
    value = PrefsGetInt(prefTerminalBackColor, defaultBackColor);
    selection = LstGetSelection(PrvGetObjectByID(PrefsFormBackColorListID));
    if (selection != value  &&  selection != noListSelection) {
        PrefsPutInt(prefTerminalBackColor, selection);
        sshChanged = true;
    }

    // term type
    // does NOT affect active ssh session
    if (PrvGetControlValue(PrefsFormXtermCheckboxID))
        PrefsPutString(prefTerminalType, "xterm");
    else if (PrvGetControlValue(PrefsFormXtermColorCheckboxID))
        PrefsPutString(prefTerminalType, "xterm-color");
    else if (PrvGetControlValue(PrefsFormVT100CheckboxID))
        PrefsPutString(prefTerminalType, "vt100");
    else if (PrvGetControlValue(PrefsFormVT220CheckboxID))
        PrefsPutString(prefTerminalType, "vt220");

    if (sshChanged) ssh_reread_prefs(ss);
}


static void InvalidateSampleText(void)
{
    EventType e2;
    e2.eType = frmGadgetMiscEvent;
    e2.data.gadgetMisc.gadgetID = PrefsFormSampleTextGadgetID;
    e2.data.gadgetMisc.gadgetP = 
        PrvGetObjectByID(e2.data.gadgetMisc.gadgetID);
    e2.data.gadgetMisc.selector = 0;
    e2.data.gadgetMisc.dataP = NULL;
    EvtAddEventToQueue(&e2);
}


Boolean PrefsFormHandleEvent(EventPtr e)
{
    FormPtr frmP = FrmGetActiveForm();

    switch (e->eType) {
    case frmOpenEvent:
        FrmSetGadgetHandler(FrmGetActiveForm(), 
                            PrvGetObjectIndexByID(PrefsFormSampleTextGadgetID),
                            SampleTextEventHandler);
        LoadPrefsForm();
        FrmDrawForm(frmP);
        return true;

    case popSelectEvent: {
        // force sample text to redraw with new colors
        InvalidateSampleText();
        return false;
    }

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case PrefsFormOKButtonID:
            // save values; leave form
            SavePrefsForm();
            FrmReturnToForm(0);
            return true;

        case PrefsFormCancelButtonID:
            // DON'T save values; leave form
            FrmReturnToForm(0);
            return true;

        // checkboxes

        case PrefsFormSmallFontCheckboxID:
            PrvSetControlValue(PrefsFormLargeFontCheckboxID, 0);
            InvalidateSampleText();
            return true;
        case PrefsFormLargeFontCheckboxID:
            PrvSetControlValue(PrefsFormSmallFontCheckboxID, 0);
            InvalidateSampleText();
            return true;

        case PrefsFormXtermCheckboxID:
            PrvSetControlValue(PrefsFormXtermCheckboxID, 1);
            PrvSetControlValue(PrefsFormXtermColorCheckboxID, 0);
            PrvSetControlValue(PrefsFormVT100CheckboxID, 0);
            PrvSetControlValue(PrefsFormVT220CheckboxID, 0);
            return true;
        case PrefsFormXtermColorCheckboxID:
            PrvSetControlValue(PrefsFormXtermCheckboxID, 0);
            PrvSetControlValue(PrefsFormXtermColorCheckboxID, 1);
            PrvSetControlValue(PrefsFormVT100CheckboxID, 0);
            PrvSetControlValue(PrefsFormVT220CheckboxID, 0);
            return true;
        case PrefsFormVT100CheckboxID:
            PrvSetControlValue(PrefsFormXtermCheckboxID, 0);
            PrvSetControlValue(PrefsFormXtermColorCheckboxID, 0);
            PrvSetControlValue(PrefsFormVT100CheckboxID, 1);
            PrvSetControlValue(PrefsFormVT220CheckboxID, 0);
            return true;
        case PrefsFormVT220CheckboxID:
            PrvSetControlValue(PrefsFormXtermCheckboxID, 0);
            PrvSetControlValue(PrefsFormXtermColorCheckboxID, 0);
            PrvSetControlValue(PrefsFormVT100CheckboxID, 0);
            PrvSetControlValue(PrefsFormVT220CheckboxID, 1);
            return true;

        default:
            return false;
        }

    case frmCloseEvent:
        return false;

    default:
        return false;
    }
}



static void DrawSampleText(RectangleType bounds)
{
    uint32_t fg_color;
    uint32_t bg_color;
    char *str = "Sample terminal text";
    Int16 width, height;
    Int16 x, y;

    WinPushDrawState();
    
    fg_color = LstGetSelection(PrvGetObjectByID(PrefsFormForeColorListID));
    bg_color = LstGetSelection(PrvGetObjectByID(PrefsFormBackColorListID));
    set_palm_color(fg_color * 2, bg_color * 2);
    
    if (PrvGetControlValue(PrefsFormSmallFontCheckboxID)) {
        FntSetFont(NanoFontID);
    } else {
        FntSetFont(MediumFontID);
    }

    width = FntCharsWidth(str, strlen(str));
    height = FntLineHeight();

    x = bounds.topLeft.x + (bounds.extent.x / 2) - (width / 2);
    y = bounds.topLeft.y + (bounds.extent.y / 2) - (height / 2);
    
    WinEraseRectangle(&bounds, 0);
    WinDrawChars(str, strlen(str), x, y);
    
    WinPopDrawState();
}


static Boolean SampleTextEventHandler(struct FormGadgetTypeInCallback *gadget, 
                                      UInt16 cmd, void *paramP)
{
    Boolean handled = false; 

    switch (cmd) { 
    case formGadgetDrawCmd: 
        // Sent to active gadgets any time form is  
        // drawn or redrawn. 
        DrawSampleText(gadget->rect);
        handled = true; 
        break; 

    case formGadgetHandleEventCmd: {
        EventPtr e = (EventPtr)paramP;
        // Sent when form receives a gadget event.  
        // paramP points to EventType structure.  
        if (e->eType == frmGadgetEnterEvent) { 
            // penDown in gadget's bounds.  
            // do nothing
            handled = true; 
        } else if (e->eType == frmGadgetMiscEvent) { 
            // This event is sent by your application 
            // when it needs to send info to the gadget 
            DrawSampleText(gadget->rect);
            handled = true;
        } 
        break; 
    }

    case formGadgetDeleteCmd:  
        // Perform any cleanup prior to deletion. 
        break; 

    case formGadgetEraseCmd:  
        // FrmHideObject takes care of this if you  
        // return false.  
        handled = false; 
        break; 

    default:
        handled = false;
        break;
    } 

    return handled; 
} 
